
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: print

local next = next
local table = table
local type = type
local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local MAJOR, MINOR = "SixQueue-1.0", 1
if not LibStub then error(MAJOR .. " requires LibStub.") end
local Library = LibStub:NewLibrary(MAJOR, MINOR)
if not Library then return end

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

-- Check if a callback queue exists for the given key.
local function Queue_Exists(queue, key)
	if not key then
		print("SixQueue: Error: A key must not be nil.")
		return
	end

	return not not queue[key]
end

-- Add a callback for the given key to the queue.
local function Queue_Add(queue, key, callback, ...)
	if not key or type(callback) ~= "function" then
		print("SixQueue: Error: A key must not be nil and a callback must be a function.")
		return
	end

	if not Queue_Exists(queue, key) then
		queue[key] = {}
	end

	table.insert(queue[key], {callback = callback, args = {...}})
end

-- Process a key's callback queue.
local function processQueueEntry(queue, key, scope)
	for _, entry in next, queue[key] do
		entry.callback(unpack(entry.args))
	end

	queue[key] = nil
end

-- Process the queue.
local function Queue_Process(queue, key, scope)
	if key then
		if Queue_Exists(queue, key) then
			processQueueEntry(queue, key, scope)
		end
	else
		for key in next, queue do
			processQueueEntry(queue, key, scope)
		end
	end
end

-------------------------------------------------------------------------------
-- Library
-------------------------------------------------------------------------------

-- Create a new queue object.
function Library:CreateQueue(scope)
	local Queue, queue = {}, {}

	function Queue:Add(key, callback, ...)
		Queue_Add(queue, key, callback, ...)
	end

	function Queue:Exists(key)
		Queue_Exists(queue, key)
	end

	function Queue:Process(key)
		Queue_Process(queue, key, scope)
	end

	return Queue
end
